"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.dotnet = exports.System = exports.None = exports.EnumType = exports.ClassType = void 0;
const codegen_1 = require("@azure-tools/codegen");
const expression_1 = require("./expression");
const namespace_1 = require("./namespace");
const parameter_1 = require("./parameter");
const property_1 = require("./property");
const linq_1 = require("@azure-tools/linq");
class ClassType {
    constructor(namespace, name) {
        this.name = name;
        this.namespace = typeof (namespace) === 'string' ? new namespace_1.Namespace(namespace) : namespace;
    }
    get fullName() {
        return this.namespace.fullName ? `${this.namespace.fullName}.${this.name}` : this.name;
    }
    get declaration() {
        return `${this.fullName}`;
    }
    toString() {
        return this.declaration;
    }
    new(...parameters) {
        return expression_1.toExpression(`new ${this.fullName}(${parameters.joinWith(expression_1.valueOf)})`);
    }
    Cast(expression) {
        return expression_1.toExpression(`(${this.declaration})${expression}`);
    }
}
exports.ClassType = ClassType;
class EnumType {
    constructor(namespace, name) {
        this.namespace = namespace;
        this.name = name;
    }
    get declaration() {
        return `${this.namespace}.${this.name}`;
    }
    member(enumMember) {
        return new expression_1.LiteralExpression(`${this.declaration}.${enumMember}`);
    }
    get definition() {
        return '';
    }
    toString() {
        return this.declaration;
    }
    newProperty(name, objectInitializer) {
        return new property_1.Property(name, this, objectInitializer);
    }
    newParameter(name, objectInitializer) {
        return new parameter_1.Parameter(name, this, objectInitializer);
    }
    withMembers() {
        /* eslint-disable */
        const $this = this;
        return new Proxy($this, {
            get(target, prop, receiver) {
                // return a property if there is one for this.
                if (target[prop]) {
                    return target[prop];
                }
                return $this.member(prop.toString());
            }
        });
        /* eslint-enable */
    }
}
exports.EnumType = EnumType;
exports.None = new namespace_1.Namespace('');
const system = new namespace_1.Namespace('global::System');
const threading = new namespace_1.Namespace('Threading', system);
const text = new namespace_1.Namespace('Text', system);
const rx = new namespace_1.Namespace('RegularExpressions', text);
const tasks = new namespace_1.Namespace('Tasks', threading);
const action = new ClassType(system, 'Action');
const collections = new namespace_1.Namespace('Collections', system);
const generic = new namespace_1.Namespace('Generic', collections);
const net = new namespace_1.Namespace('Net', system);
const io = new namespace_1.Namespace('IO', system);
const http = new namespace_1.Namespace('Http', net);
const headers = new namespace_1.Namespace('Headers', http);
const task = new ClassType(tasks, 'Task');
const encoding = new ClassType(text, 'Encoding');
const linq = new namespace_1.Namespace('Linq', system);
const xml = new namespace_1.Namespace('Xml', system);
const xmllinq = new namespace_1.Namespace('Linq', xml);
const stringClass = new ClassType(system, 'String');
exports.System = codegen_1.intersect(system, {
    Threading: codegen_1.intersect(threading, {
        CancellationToken: new ClassType(threading, 'CancellationToken'),
        CancellationTokenSource: new ClassType(threading, 'CancellationTokenSource'),
        Tasks: codegen_1.intersect(tasks, {
            Task(taskType) {
                return taskType ? new ClassType(tasks, `Task<${taskType.declaration}>`) : task;
            }
        })
    }),
    Convert: codegen_1.intersect(new ClassType(system, 'Convert'), {
        ToBase64String: (expression) => expression_1.toExpression(`${exports.System.Convert}.ToBase64String( ${expression})`)
    }),
    Linq: codegen_1.intersect(linq, {
        Enumerable: codegen_1.intersect(new ClassType(linq, 'Enumerable'), {
            ToArray: (source) => expression_1.toExpression(`${exports.System.Linq.Enumerable}.ToArray(${expression_1.valueOf(source)})`),
            Select: (source, selector) => expression_1.toExpression(`${exports.System.Linq.Enumerable}.Select(${expression_1.valueOf(source)}, ${selector})`),
            Where: (source, predicate) => expression_1.toExpression(`${exports.System.Linq.Enumerable}.Where(${expression_1.valueOf(source)}, ${predicate})`),
            ToDictionary: (source, keySelector, valueSelector) => expression_1.toExpression(`${exports.System.Linq.Enumerable}.ToDictionary(${expression_1.valueOf(source)}, ${keySelector}, ${valueSelector})`),
            Empty: (type) => expression_1.toExpression(`${exports.System.Linq.Enumerable}.Empty<${type.declaration}>()`),
        })
    }),
    Object: new ClassType(system, 'Object'),
    String: codegen_1.intersect(stringClass, {
        Empty: new expression_1.LiteralExpression('global::System.String.Empty'),
        IsNullOrEmpty: (expression) => expression_1.toExpression(`${exports.System.String}.IsNullOrEmpty(${expression_1.toExpression(expression)})`),
        IsNullOrWhitespace: (expression) => expression_1.toExpression(`${exports.System.String}.IsNullOrWhitespace(${expression_1.toExpression(expression)})`),
        /** @description Binds a Variable to the known instance methods of this type */
        $BindTo: ($instance) => {
            return codegen_1.intersect($instance, {
                Substring: (start, end) => {
                    return end ? `${$instance}.Substring(${start},${end})` : `${$instance}.Substring(${start})`;
                }
            });
        },
    }),
    StringComparison: new EnumType(system, 'StringComparison'),
    DateTime: new ClassType(system, 'DateTime'),
    DateTimeKind: codegen_1.intersect(new ClassType(system, 'DateTimeKind'), {
        Utc: new expression_1.LiteralExpression('global::System.DateTimeKind.Utc'),
    }),
    EventArgs: new ClassType(system, 'EventArgs'),
    Exception: new ClassType(system, 'Exception'),
    OperationCanceledException: new ClassType(system, 'OperationCanceledException'),
    AggregateException: new ClassType(system, 'AggregateException'),
    TimeSpan: new ClassType(system, 'TimeSpan'),
    Type: new ClassType(system, 'Type'),
    Uri: new ClassType(system, 'Uri'),
    IFormatProvider: new ClassType(system, 'IFormatProvider'),
    Xml: codegen_1.intersect(xml, {
        Linq: codegen_1.intersect(xmllinq, {
            XElement: new ClassType(xmllinq, 'XElement'),
            XAttribute: new ClassType(xmllinq, 'XAttribute')
        })
    }),
    IO: codegen_1.intersect(io, {
        Stream: new ClassType(io, 'Stream')
    }),
    Text: codegen_1.intersect(text, {
        Encoding: codegen_1.intersect(encoding, {
            UTF8: new expression_1.LiteralExpression(`${encoding.declaration}.UTF8`)
        }),
        RegularExpressions: codegen_1.intersect(rx, {
            Regex: new ClassType(rx, 'Regex')
        })
    }),
    Net: codegen_1.intersect(net, {
        WebProxy: new ClassType(net, 'WebProxy'),
        Http: codegen_1.intersect(http, {
            HttpRequestMessage: new ClassType(http, 'HttpRequestMessage'),
            HttpClient: new ClassType(http, 'HttpClient'),
            HttpClientHandler: new ClassType(http, 'HttpClientHandler'),
            HttpResponseMessage: new ClassType(http, 'HttpResponseMessage'),
            Headers: codegen_1.intersect(headers, {
                MediaTypeHeaderValue: codegen_1.intersect(new ClassType(headers, 'MediaTypeHeaderValue'), {
                    Parse: (header) => expression_1.toExpression(`${exports.System.Net.Http.Headers.MediaTypeHeaderValue}.Parse("${header}")`)
                }),
                HttpHeaders: new ClassType(headers, 'HttpHeaders'),
                HttpResponseHeaders: new ClassType(headers, 'HttpResponseHeaders'),
            }),
            StringContent: new ClassType(http, 'StringContent'),
            StreamContent: new ClassType(http, 'StreamContent'),
            MultipartFormDataContent: new ClassType(http, 'MultipartFormDataContent'),
        }),
        /* eslint-disable */
        HttpStatusCode: codegen_1.intersect(new ClassType(net, 'HttpStatusCode'), {
            default: new expression_1.LiteralExpression(''),
            100: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Continue'),
            101: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.SwitchingProtocols'),
            200: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.OK'),
            201: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Created'),
            202: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Accepted'),
            203: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NonAuthoritativeInformation'),
            204: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NoContent'),
            205: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.ResetContent'),
            206: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.PartialContent'),
            300: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Ambiguous'),
            301: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Moved'),
            302: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Redirect'),
            303: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.SeeOther'),
            304: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NotModified'),
            305: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.UseProxy'),
            306: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Unused'),
            307: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.TemporaryRedirect'),
            400: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.BadRequest'),
            401: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Unauthorized'),
            402: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.PaymentRequired'),
            403: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Forbidden'),
            404: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NotFound'),
            405: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.MethodNotAllowed'),
            406: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NotAcceptable'),
            407: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.ProxyAuthenticationRequired'),
            408: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.RequestTimeout'),
            409: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Conflict'),
            410: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Gone'),
            411: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.LengthRequired'),
            412: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.PreconditionFailed'),
            413: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.RequestEntityTooLarge'),
            414: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.RequestUriTooLong'),
            415: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.UnsupportedMediaType'),
            416: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.RequestedRangeNotSatisfiable'),
            417: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.ExpectationFailed'),
            426: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.UpgradeRequired'),
            500: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.InternalServerError'),
            501: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NotImplemented'),
            502: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.BadGateway'),
            503: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.ServiceUnavailable'),
            504: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.GatewayTimeout'),
            505: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.HttpVersionNotSupported'),
            Continue: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Continue'),
            SwitchingProtocols: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.SwitchingProtocols'),
            OK: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.OK'),
            Created: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Created'),
            Accepted: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Accepted'),
            NonAuthoritativeInformation: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NonAuthoritativeInformation'),
            NoContent: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NoContent'),
            ResetContent: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.ResetContent'),
            PartialContent: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.PartialContent'),
            Ambiguous: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Ambiguous'),
            Moved: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Moved'),
            Redirect: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Redirect'),
            SeeOther: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.SeeOther'),
            NotModified: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NotModified'),
            UseProxy: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.UseProxy'),
            Unused: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Unused'),
            TemporaryRedirect: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.TemporaryRedirect'),
            BadRequest: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.BadRequest'),
            Unauthorized: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Unauthorized'),
            PaymentRequired: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.PaymentRequired'),
            Forbidden: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Forbidden'),
            NotFound: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NotFound'),
            MethodNotAllowed: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.MethodNotAllowed'),
            NotAcceptable: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NotAcceptable'),
            ProxyAuthenticationRequired: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.ProxyAuthenticationRequired'),
            RequestTimeout: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.RequestTimeout'),
            Conflict: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Conflict'),
            Gone: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.Gone'),
            LengthRequired: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.LengthRequired'),
            PreconditionFailed: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.PreconditionFailed'),
            RequestEntityTooLarge: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.RequestEntityTooLarge'),
            RequestUriTooLong: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.RequestUriTooLong'),
            UnsupportedMediaType: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.UnsupportedMediaType'),
            RequestedRangeNotSatisfiable: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.RequestedRangeNotSatisfiable'),
            ExpectationFailed: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.ExpectationFailed'),
            UpgradeRequired: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.UpgradeRequired'),
            InternalServerError: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.InternalServerError'),
            NotImplemented: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.NotImplemented'),
            BadGateway: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.BadGateway'),
            ServiceUnavailable: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.ServiceUnavailable'),
            GatewayTimeout: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.GatewayTimeout'),
            HttpVersionNotSupported: new expression_1.LiteralExpression('global::System.Net.HttpStatusCode.HttpVersionNotSupported')
        }),
    }),
    Collections: codegen_1.intersect(collections, {
        Hashtable: new ClassType(collections, 'Hashtable'),
        IDictionary: new ClassType(collections, 'IDictionary'),
        IEnumerator: new ClassType(collections, 'IEnumerator'),
        Generic: codegen_1.intersect(generic, {
            Dictionary(keyType, valueType) {
                return new ClassType(generic, `Dictionary<${keyType.declaration},${valueType.declaration}>`);
            },
            IDictionary(keyType, valueType) {
                return {
                    declaration: `${generic.fullName}.IDictionary<${keyType.declaration},${valueType.declaration}>`,
                    allProperties: []
                };
            },
            KeyValuePair(keyType, valueType) {
                return new ClassType(generic, `KeyValuePair<${keyType.declaration},${valueType.declaration}>`);
            },
            IEnumerable(type) {
                return new ClassType(generic, `IEnumerable<${type.declaration}>`);
            },
            HashSet(type) {
                return new ClassType(generic, `HashSet<${type.declaration}>`);
            },
            IEnumerator(type) {
                return new ClassType(generic, `IEnumerator<${type.declaration}>`);
            },
            ICollection(type) {
                return new ClassType(generic, `ICollection<${type.declaration}>`);
            }
        })
    }),
    Action(...actionParameters) {
        return linq_1.length(actionParameters) === 0 ? action : new ClassType(system, `Action<${actionParameters.filter(each => each.declaration).joinWith(each => each.declaration)}>`);
    },
    Func(...funcParameters) {
        return new ClassType(system, `Func<${funcParameters.joinWith(each => each.declaration)}>`);
    }
});
exports.dotnet = {
    Unknown: new ClassType(exports.None, 'null'),
    ToDo: new ClassType(exports.None, 'null'),
    Void: new ClassType(exports.None, 'void'),
    String: new ClassType(exports.None, 'string'),
    StringArray: new ClassType(exports.None, 'string[]'),
    Int: new ClassType(exports.None, 'int'),
    Long: new ClassType(exports.None, 'long'),
    Double: new ClassType(exports.None, 'double'),
    Float: new ClassType(exports.None, 'float'),
    Binary: new ClassType(exports.None, 'byte[]'),
    Bool: new ClassType(exports.None, 'bool'),
    Object: new ClassType(exports.None, 'object'),
    Dynamic: new ClassType(exports.None, 'dynamic'),
    ThisObject: new ClassType(exports.None, 'this object'),
    Var: new ClassType(exports.None, 'var'),
    True: new expression_1.LiteralExpression('true'),
    False: new expression_1.LiteralExpression('false'),
    Null: new expression_1.LiteralExpression('null'),
    This: new expression_1.LiteralExpression('this'),
    Array: (type) => ({ declaration: `${type.declaration}[]` })
};
//# sourceMappingURL=dotnet.js.map