"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.IsDeclaration = exports.Is = exports.toExpression = exports.LambdaExpression = exports.Lambda = exports.IsExpressionDeclaration = exports.IsExpression = exports.LiteralExpression = exports.StringExpression = exports.BaseExpression = exports.And = exports.Or = exports.isAnExpression = exports.valueOf = void 0;
const codegen_1 = require("@azure-tools/codegen");
const access_modifier_1 = require("./access-modifier");
const comparisons_1 = require("./comparisons");
const statement_1 = require("./statements/statement");
const variable_1 = require("./variable");
function valueOf(expression) {
    return typeof expression === 'string' ? expression : expression.value;
}
exports.valueOf = valueOf;
function isAnExpression(item) {
    return item && item.value ? true : false;
}
exports.isAnExpression = isAnExpression;
function Or(expression1, expression2) {
    return `(${expression1} || ${expression2})`;
}
exports.Or = Or;
function And(expression1, expression2) {
    return `(${expression1} && ${expression2})`;
}
exports.And = And;
class BaseExpression {
    toString() {
        return this.value;
    }
    get IsNull() {
        return comparisons_1.IsNull(this);
    }
    get IsNotNull() {
        return comparisons_1.IsNotNull(this);
    }
    Cast(toType) {
        return comparisons_1.Cast(this, toType);
    }
}
exports.BaseExpression = BaseExpression;
/** an expression of a constant string value  */
class StringExpression extends BaseExpression {
    constructor(value) {
        super();
        this.value = `@"${value.replace(/"/g, '""')}"`;
    }
}
exports.StringExpression = StringExpression;
/** an arbitrary user-defined expression  */
class LiteralExpression extends BaseExpression {
    constructor(value) {
        super();
        this.value = value;
    }
}
exports.LiteralExpression = LiteralExpression;
/** a c# 'is' expression */
class IsExpression extends BaseExpression {
    constructor(expression, isType) {
        super();
        this.expression = expression;
        this.isType = isType;
    }
    get value() {
        return `${valueOf(this.expression)} is ${this.isType.declaration}`;
    }
}
exports.IsExpression = IsExpression;
/** a c# 'is' expression that declares a local variable  */
class IsExpressionDeclaration extends variable_1.LocalVariable {
    constructor(expression, isType, name) {
        super(name, isType);
        this.expression = expression;
        this.isType = isType;
    }
    get check() {
        return new LiteralExpression(`${valueOf(this.expression)} is ${this.isType.declaration} ${this.name}`);
    }
    get IsNull() {
        return comparisons_1.IsNull(this);
    }
    get IsNotNull() {
        return comparisons_1.IsNotNull(this);
    }
    Cast(toType) {
        return comparisons_1.Cast(this, toType);
    }
    get value() {
        return this.name;
    }
    toString() {
        return this.value;
    }
}
exports.IsExpressionDeclaration = IsExpressionDeclaration;
function Lambda(parameters, body, objectIntializer) {
    throw new Error('not implemented');
}
exports.Lambda = Lambda;
class LambdaExpression extends statement_1.Statements {
    constructor(parameters, body, objectIntializer) {
        super(body);
        this.parameters = parameters;
        this.body = body;
        this.async = access_modifier_1.Modifier.None;
        this.apply(objectIntializer);
    }
    get value() {
        return `
${this.async}(${this.parameters.joinWith(each => each.name)}) => {
${codegen_1.indent(super.implementation)}
}
`.trim();
    }
    toString() {
        return this.value;
    }
    get IsNull() {
        return comparisons_1.IsNull(this);
    }
    Cast(toType) {
        return comparisons_1.Cast(this, toType);
    }
    get IsNotNull() {
        return comparisons_1.IsNotNull(this);
    }
}
exports.LambdaExpression = LambdaExpression;
function toExpression(expression) {
    return typeof expression === 'string' ? new LiteralExpression(expression) : expression;
}
exports.toExpression = toExpression;
/** a c# 'is' expression */
function Is(expression, isType) {
    return new IsExpression(expression, isType);
}
exports.Is = Is;
/** a c# 'is' expression that declares a local variable  */
function IsDeclaration(expression, isType, name) {
    return new IsExpressionDeclaration(expression, isType, name);
}
exports.IsDeclaration = IsDeclaration;
//# sourceMappingURL=expression.js.map