"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.Namespace = void 0;
const codegen_1 = require("@azure-tools/codegen");
const codegen_2 = require("@azure-tools/codegen");
const class_1 = require("./class");
const import_1 = require("./import");
const interface_1 = require("./interface");
class Namespace extends codegen_1.Initializer {
    constructor(name, parent, objectInitializer) {
        super();
        this.name = name;
        this.parent = parent;
        this.usings = new Array();
        this.classes = new Array();
        this.interfaces = new Array();
        this.delegates = new Array();
        this.namespaces = new Array();
        this.header = '';
        this.folder = this.fullName.replace(/\./g, '/');
        this.apply(objectInitializer);
    }
    get outputFolder() {
        return this.folder;
    }
    addImport(using) {
        if (this.usings.indexOf(using) === -1) {
            this.usings.push(using);
        }
        return using;
    }
    addClass(c) {
        if (this.classes.indexOf(c) === -1) {
            const found = this.classes.find(each => each.name === c.name && each.fileName === c.fileName);
            if (found) {
                //console.error(`ERROR: Class ${c.name} already exists in namespace${this.name}`);
                throw new Error(`ERROR: Class ${c.fullName} already exists in namespace ${this.fullName}`);
            }
            this.classes.push(c);
        }
        return c;
    }
    addInterface(i) {
        if (this.interfaces.indexOf(i) === -1) {
            this.interfaces.push(i);
        }
        return i;
    }
    addDelegate(delegate) {
        if (this.delegates.indexOf(delegate) === -1) {
            this.delegates.push(delegate);
        }
        return delegate;
    }
    addNamespace(n) {
        if (this.namespaces.indexOf(n) === -1) {
            this.namespaces.push(n);
        }
        return n;
    }
    add(item) {
        if (item instanceof class_1.Class) {
            this.addClass(item);
            return item;
        }
        if (item instanceof Namespace) {
            this.addNamespace(item);
            return item;
        }
        if (item instanceof interface_1.Interface) {
            this.addInterface(item);
            return item;
        }
        if (item instanceof import_1.ImportDirective) {
            this.addImport(item);
            return item;
        }
        throw Error(`FATAL - UNABLE TO ADD UNKNOWN TYPE for '${JSON.stringify(item)}'`);
    }
    findClassByName(name) {
        return this.classes.filter(each => each.name === name);
    }
    get fullName() {
        if (this.parent instanceof Namespace) {
            return codegen_2.dotCombine(this.parent.fullName, this.name);
        }
        return this.name;
    }
    toString() {
        return this.fullName;
    }
    async writeFiles(writer) {
        // write out all the files
        const w = (filename, content) => writer(filename, content.replace(/\/\*-.*?-\*\/\s*/g, ''));
        // handle nested namespaces
        const children = this.namespaces.map(async (namespace) => namespace.writeFiles(w));
        // combine class (XYZ) and interfaces (IXYZ) together in a single file
        const classes = codegen_2.toMap(this.classes, c => c.fileName);
        const interfaces = codegen_2.toMap(this.interfaces, i => i.fileName);
        for (const [key, classesWithSameName] of classes) {
            const contents = classesWithSameName.map(each => each.definition);
            for (const interfaceName of [`I${key}`, `I${key}Internal`]) { // pick up interfaces and interfacesInternal in the same file. 
                const interfacesWithSameName = interfaces.get(interfaceName);
                if (interfacesWithSameName) {
                    contents.push(...interfacesWithSameName.map(each => each.definition));
                    // remove from the list.
                    interfaces.delete(interfaceName);
                }
            }
            await w(`${this.outputFolder}/${key}.cs`, this.render(contents.join(codegen_2.EOL)));
        }
        for (const [key, interfacesWithSameName] of interfaces) {
            const contents = interfacesWithSameName.map(each => each.definition);
            await w(`${this.outputFolder}/${key}.cs`, this.render(contents.join(codegen_2.EOL)));
        }
        // do the delegates in a single file
        const delegates = this.delegates.map(v => v.implementation).join(codegen_2.EOL);
        if (delegates) {
            await w(`${this.outputFolder}/delegates.cs`, this.render(delegates));
        }
        // wait for children to finish.
        await (Promise.all(children));
    }
    render(content) {
        // all files get the header comment
        const header = codegen_2.comment(this.header);
        // all files get imports
        const imports = codegen_2.indent(this.usings.map((v) => v.implementation).join(codegen_2.EOL), 1);
        const body = codegen_2.indent(content, 1);
        return `
${header}

namespace ${this.fullName}
{
${imports}

${body}
}
`.trim().replace(/ *$/gm, '').replace(/\n\n+/g, '\n\n').replace(/^\s*EOL\s*$/igm, '');
    }
}
exports.Namespace = Namespace;
//# sourceMappingURL=namespace.js.map