"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.BackedProperty = exports.LazyProperty = exports.LambdaProperty = exports.Indexer = exports.Property = void 0;
const codegen_1 = require("@azure-tools/codegen");
const codegen_2 = require("@azure-tools/codegen");
const linq_1 = require("@azure-tools/linq");
const access_modifier_1 = require("./access-modifier");
const attribute_1 = require("./attribute");
const doc_comments_1 = require("./doc-comments");
const expression_1 = require("./expression");
const statement_1 = require("./statements/statement");
const variable_1 = require("./variable");
class Property extends variable_1.Variable {
    constructor(name, type, objectInitializer) {
        super();
        this.name = name;
        this.type = type;
        this['new'] = access_modifier_1.Modifier.None;
        this.getAccess = access_modifier_1.Access.Public;
        this.setAccess = access_modifier_1.Access.Public;
        this['static'] = access_modifier_1.Modifier.None;
        this.virtual = access_modifier_1.Modifier.None;
        this.sealed = access_modifier_1.Modifier.None;
        this.override = access_modifier_1.Modifier.None;
        this.abstract = access_modifier_1.Modifier.None;
        this.extern = access_modifier_1.Modifier.None;
        this.attributes = new Array();
        this.metadata = {};
        this.description = '';
        name = name.trim();
        this.apply(objectInitializer);
        if (!this.description.trim()) {
            this.description = '';
        }
    }
    get visibility() {
        return access_modifier_1.highestAccess(this.getAccess, this.setAccess) || access_modifier_1.Access.Explicit;
    }
    add(item) {
        if (item instanceof attribute_1.Attribute) {
            this.attributes.push(item);
            return item;
        }
        throw Error(`FATAL - UNABLE TO ADD UNKNOWN TYPE for '${JSON.stringify(item)}'`);
    }
    get attributeDeclaration() {
        return linq_1.length(this.attributes) > 0 ? `${this.attributes.joinWith(each => `${each.value}`, codegen_2.EOL)}${codegen_2.EOL}` : '';
    }
    get getterDeclaration() {
        return this.getAccess === this.visibility ? 'get' : `${this.getAccess} get`;
    }
    get setterDeclaration() {
        return this.setAccess === this.visibility ? 'set' : `${this.setAccess} set`;
    }
    get getter() {
        if (!this.get) {
            // if there is a set expression/body then this can't bet auto
            return this.set ? '' : `${this.getterDeclaration};`;
        }
        if (expression_1.isAnExpression(this.get)) {
            return `${this.getterDeclaration} => ${expression_1.valueOf(this.get)};`;
        }
        const gi = new statement_1.Statements(this.get).implementation;
        return gi.indexOf('\n') == -1 ? `${this.getterDeclaration} { ${gi} }` :
            `${this.getterDeclaration}
{
${codegen_2.indent(gi, 2)}
}
`;
    }
    get setter() {
        if (!this.set) {
            // if there is a get expression/body then this can't bet auto
            return this.get ? '' : `${this.setterDeclaration};`;
        }
        if (expression_1.isAnExpression(this.set)) {
            return `${this.setterDeclaration} => ${expression_1.valueOf(this.set)};`;
        }
        const si = new statement_1.Statements(this.set).implementation;
        return si.indexOf('\n') == -1 ? `${this.setterDeclaration} { ${si} }` :
            `${this.setterDeclaration}
{
${codegen_2.indent(si, 2)}
}
`;
    }
    get declaration() {
        const s = this.setter;
        const g = this.getter;
        const decl = `
${codegen_2.docComment(doc_comments_1.summary(this.description))}
${this.attributeDeclaration}${this.new}${this.visibility} ${this.static} ${this.virtual} ${this.sealed} ${this.override} ${this.abstract} ${this.extern} ${this.type.declaration} ${this.name}`;
        if (g && s) {
            return (g.indexOf('\n') > -1 || s.indexOf('\n') > -1) ?
                // at least one is more that one line
                `${decl}
{
${codegen_2.indent(g, 2)}
${codegen_2.indent(s, 2)}
}`.slim() :
                // both are single line 
                `${decl} { ${g} ${s} }`.slim();
        }
        if (s) {
            // no getter?
            return (s.indexOf('\n') > -1) ?
                `${decl}
{
${codegen_2.indent(s, 2)}
}`.slim() :
                // both are single line 
                `${decl} { ${s} }`.slim();
        }
        if (g) {
            // no setter
            return (g.indexOf('\n') > -1) ?
                `${decl}
{
${codegen_2.indent(g, 2)}
}`.slim() :
                // single line 
                `${decl} { ${g} }`.slim();
        }
        return `${decl} { get; set; }`.slim();
    }
    get value() {
        return `${this.name}`.trim();
    }
    get valuePrivate() {
        return `${this.name}`.trim();
    }
    assign(expression) {
        return `${this.name} = ${expression_1.valueOf(expression)};`;
    }
    assignPrivate(expression) {
        return this.assign(expression);
    }
    get declarationExpression() {
        return this;
    }
    get declarationStatement() {
        throw new Error('Property can not be a declaration statement');
    }
    invokeMethod(methodName, ...parameters) {
        const e = `${this.value}.${methodName}(${parameters.joinWith(expression_1.valueOf)})`;
        return codegen_1.intersect(expression_1.toExpression(e), {
            implementation: `${e};`
        });
    }
}
exports.Property = Property;
class Indexer extends Property {
    constructor(keyType, valueType, objectInitializer) {
        super(`this[${keyType.declaration} index]`, valueType);
        this.keyType = keyType;
        this.apply(objectInitializer);
    }
    get index() {
        return 'index';
    }
}
exports.Indexer = Indexer;
class LambdaProperty extends Property {
    constructor(name, type, expression, objectInitializer) {
        super(name, type);
        this.name = name;
        this.type = type;
        this.expression = expression;
        this.apply(objectInitializer);
    }
    get declaration() {
        return `
${codegen_2.docComment(doc_comments_1.summary(this.description))}
${this.attributeDeclaration}${this.new}${this.visibility} ${this.static} ${this.virtual} ${this.sealed} ${this.override} ${this.abstract} ${this.extern} ${this.type.declaration} ${this.name} => ${expression_1.valueOf(this.expression)};
`.slim();
    }
}
exports.LambdaProperty = LambdaProperty;
class LazyProperty extends Property {
    constructor(name, type, expression, objectInitializer) {
        super(name, type);
        this.name = name;
        this.type = type;
        this.expression = expression;
        this.instanceAccess = 'this';
        this.backingName = `_${this.name.uncapitalize()}`;
        this.apply(objectInitializer);
    }
    get declaration() {
        return `
${codegen_2.docComment(doc_comments_1.summary(`Backing field for <see cref="${this.name}" /> property.`))}
private ${this.static} ${this.type.declaration} ${this.backingName};
EOL
${codegen_2.docComment(doc_comments_1.summary(this.description))}
${this.attributeDeclaration}${this.new}${this.visibility} ${this.static} ${this.virtual} ${this.sealed} ${this.override} ${this.abstract} ${this.extern} ${this.type.declaration} ${this.name} => ${this.instanceAccess}.${this.backingName}?? (${this.instanceAccess}.${this.backingName} = ${this.expression.value});
`.slim();
    }
}
exports.LazyProperty = LazyProperty;
class BackedProperty extends Property {
    constructor(name, type, objectInitializer) {
        const backingName = `_${name.uncapitalize()}`;
        super(name, type, {
            get: expression_1.toExpression(`this.${backingName}`),
            set: expression_1.toExpression(`this.${backingName} = value`)
        });
        this.backingName = backingName;
        this.apply(objectInitializer);
    }
    get declaration() {
        return `
${codegen_2.docComment(doc_comments_1.summary(`Backing field for <see cref="${this.name}" /> property.`))}
private ${this.type.declaration} ${this.backingName}${this.initializer ? `= ${expression_1.valueOf(this.initializer)}` : ''};
EOL
${super.declaration}
`.trim();
        // ${docComment(summary(this.description))}
        // ${ this.attributeDeclaration } ${ this.new } ${ this.visibility } ${ this.static } ${ this.virtual } ${ this.sealed } ${ this.override } ${ this.abstract } ${ this.extern } ${ this.type.declaration } ${ this.name } { ${ this.getterDeclaration } { return this.${ this.backingName }; } ${ this.setterDeclaration } { this.${ this.backingName } = value; } }
        // EOL
    }
    get value() {
        return `${this.name}`;
    }
    get valuePrivate() {
        return `this.${this.backingName}`;
    }
    assignPrivate(expression) {
        return `{${this.backingName} = ${expression_1.valueOf(expression)};}`;
    }
}
exports.BackedProperty = BackedProperty;
//# sourceMappingURL=property.js.map