"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.Local = exports.MemberVariable = exports.LocalVariable = exports.Variable = void 0;
const codegen_1 = require("@azure-tools/codegen");
const comparisons_1 = require("./comparisons");
const expression_1 = require("./expression");
const statement_1 = require("./statements/statement");
class Variable extends codegen_1.Initializer {
    get IsNull() {
        return comparisons_1.IsNull(this);
    }
    toString() {
        return this.value;
    }
    get IsNotNull() {
        return comparisons_1.IsNotNull(this);
    }
    Cast(toType) {
        return comparisons_1.Cast(this, toType);
    }
    /**
     * Binds instance methods to this variable so they can be used as {@link Statement}
     * @param toType The ClassType definition to bind to
     */
    As(toType) {
        return toType.$BindTo(this);
    }
}
exports.Variable = Variable;
/** represents a locally declared variable */
class LocalVariable extends Variable {
    constructor(name, type, objectIntializer) {
        super();
        this.name = name;
        this.type = type;
        this.apply(objectIntializer);
    }
    get value() {
        return `${this.name}`;
    }
    get implementation() {
        return this.declarationStatement.implementation;
    }
    get declarationStatement() {
        return statement_1.toStatement(`${this.declarationExpression};`);
    }
    get declarationExpression() {
        return expression_1.toExpression(`${this.type.declaration} ${this.name}${this.initializer ? ` = ${expression_1.valueOf(this.initializer)}` : ''}`);
    }
    assign(expression) {
        return `${this.name} = ${expression_1.valueOf(expression)};`;
    }
    assignPrivate(expression) {
        return this.assign(expression);
    }
    invokeMethod(methodName, ...parameters) {
        const e = `${expression_1.valueOf(this)}.${methodName}(${parameters.joinWith(expression_1.valueOf)})`;
        return codegen_1.intersect(expression_1.toExpression(e), {
            implementation: `${e};`
        });
    }
    member(memberName) {
        /* eslint-disable */
        return new MemberVariable(this, memberName);
    }
}
exports.LocalVariable = LocalVariable;
class MemberVariable extends LocalVariable {
    constructor(variable, memberName, objectIntializer) {
        super(`${variable}.${memberName}`, { declaration: 'var' });
        this.variable = variable;
        this.memberName = memberName;
        this.apply(objectIntializer);
    }
}
exports.MemberVariable = MemberVariable;
function Local(name, initializer = 'null', type = { declaration: 'var' }) {
    return new LocalVariable(name, type, { initializer });
}
exports.Local = Local;
//# sourceMappingURL=variable.js.map